######################################################################################
# 
# ps_ms03.ps1 - "PS MS03" v1.0
# 
# Written by: Christian @ IntegrationAdmin.com
# 
# Description: This PowerShell script dynamically uses IBM's MS03 supportpac
# 
# For information, comments, and/or questions please visit my website at:
# http://www.integrationadmin.com
#
# Feel free to change this script in any way you. But please let this comment section remain, thank you!
#
######################################################################################

# Log various events
Function log ([string]$Msg, [string]$IsErrMsg){
	if ($Msg) {
		# Create a timestamp
		$Now = Get-Date
		[string]$Now = $Now
		
		# If msg is of type error
		If ($IsErrMsg -eq "error"){
			# Combine timestamp, msgtype, and actual msg
			$Msg = $Now + " ERROR! " + $Msg
			# Show result in cmd, using a specific color
			Write-Host $Msg -foregroundcolor "Red"
					
			# Append the msg to file
			$Msg | Out-File -filepath $MainLog -append
			
			# If saveqmgrc is currently being used with a qmgr then $CurrLog is not null
			# and the events from that run should be logged
			If ($CurrLog){
				$Msg | Out-File -filepath $CurrLog -append
			}
		# Else msg is of type info
		} Else {
			# Combine timestamp, msgtype, and actual msg
			$Msg = $Now + " INFO: " + $Msg
			
			# Show result in cmd, using a specific color
			Write-Host $Msg -foregroundcolor "Green"
			
			# Append the msg to file
			$Msg | Out-File -filepath $MainLog -append
			
			# If saveqmgrc is currently being used with a qmgr then $CurrLog is not null
			# and the events from that run should be logged
			If ($CurrLog){
				$Msg | Out-File -filepath $CurrLog -append
			}
		}
	}
}

function delete-files-and-folders-older-than-x-days {
	try {

		# Get current date
		$Now = Get-Date 
		# Define amount of days
		$Days = "90"
		# Define folder where files are located
		$TargetFolder = ".\saved_qmgrs"
		# Define extension to look for
		$Extension = "*.*"
		# Define LastWriteTime parameter based on $Days
		$LastWrite = $Now.AddDays(-$Days)

		# Log confirmation that delete-files-and-folders-older-than-x-days is running and how old files it's looking for
		$Tmp = " ### ATTEMPTING TO DELETE FILES AND FOLDERS OLDER THAN " + $Days.ToUpper() + " DAYS, WITH FILE EXTENSIONS " + $Extension.ToUpper() + " ###`r`n"
		log $Tmp
		
		# Get files based on lastwrite filter and specified folder
		$Files = Get-Childitem $TargetFolder -Include $Extension -Recurse | Where {$_.LastWriteTime -lt "$LastWrite"}
		
		# For each file defined in $Files
		foreach ($File in $Files){
			# If file exist and is not a folder
			if ($File -ne $null -and !$File.PSIsContainer){
				# Try to delete a file
				try {
					# Log confirmation that a file is being deleted
					log "Attempting to delete file: $File`r`n"
					# Remove the the file
					Remove-Item -recurse $File.FullName | out-null
				}
				# If something goes wrong when deleting the file
				catch {
					$ErrorMsg = "Unable to delete file: $File`r`n"
					$ErrorMsg += "Application report the following reason(s):`r`n"
					$ErrorMsg += [string]$error[0] + "`r`n`r`n"
					$ErrorMsg += [string]$error[0].InvocationInfo.PositionMessage + "`r`n`r`n"
					log $ErrorMsg "error"
				}
			}
		}
		
		# Get each item under the current directory recursively
		foreach($childItem in (Get-ChildItem $Path -Recurse))
		{
			# if it's a folder AND does not have child items of its own
			if( ($childItem.PSIsContainer) -and (!(Get-ChildItem -Recurse -Path $childItem.FullName)))
			{
				# Try to delete a folder
				try {
					# Get the full name of the current folder
					[string]$CurrentFolder = $childItem.FullName
					# Log confirmation that a folder is being deleted
					log "Attempting to delete empty folder: $CurrentFolder`r`n"
					# Deletes the current folder
					Remove-Item $childItem.FullName -Confirm:$false
				}
				# If something goes wrong when deleting a folder
				catch {
					$ErrorMsg = "Unable to delete empty folder: $CurrentFolder`r`n"
					$ErrorMsg += "Application report the following reason(s):`r`n"
					$ErrorMsg += [string]$error[0] + "`r`n`r`n"
					$ErrorMsg += [string]$error[0].InvocationInfo.PositionMessage + "`r`n`r`n"
					log $ErrorMsg "error"
				}
			}
		}
		# Log confirmation that delete-files-and-folders-older-than-x-days is running and how old files it's looking for
		$Tmp = " ### DONE DELETING FILES AND FOLDERS OLDER THAN " + $Days.ToUpper() + " DAYS, WITH FILE EXTENSIONS " + $Extension.ToUpper() + " ###`r`n"
		log $Tmp
	}
	# If something goes wrong when executing delete-files-and-folders-older-than-x-days
	catch {
		$ErrorMsg = "Error when executing function delete-files-and-folders-older-than-x-days`r`n"
		$ErrorMsg += [string]$error[0] + "`r`n`r`n"
		$ErrorMsg += [string]$error[0].InvocationInfo.PositionMessage + "`r`n`r`n"
		log $ErrorMsg "error"
	}
}

# Main function of this script. Will run saveqmgrc remotly on defined qmgrs in qmgrs.xml
function start-saveqmgrc {
	try {
		# Get the content from qmgrs.xml
		[xml] $XmlQmgrs = Get-Content ".\config\qmgrs.xml"

		# Define the Xpath that will be used for qmgrs.xml
		$Xpath = "/Qmgrs/Qmgr"

		# Iterate the result
		$XmlQmgrs | select-xml -xpath $Xpath | foreach {
			# Store the name of the qmgr currently in use
			$Qmgr = $_.node.Name
			# Store the channel currently in use
			$Channel = $_.node.Channel
			# Store the serverhost currently in use
			$ServerHost = $_.node.ServerHost
			# Store the port currently in use
			$Port = $_.node.Port
			# Store the mca user currently in use, which automaticly is the account running this script
			$Mca = $env:username
			# Store the optionsfile currently in use
			$OptionsFile = $_.node.OptionsFile
			# Store the current time
			$CurrDate = Get-Date
			# Create a custom timestamp based on $CurrDate
			[string]$CurrTimeStamp = $CurrDate.Year.ToString() + "-" + $CurrDate.Month.ToString() + "-" + $CurrDate.Day.ToString() + "_" + $CurrDate.Hour.ToString() + "." + $CurrDate.Minute.ToString() + "." + $CurrDate.Second.ToString() + $CurrDate.millisecond.ToString()
			# Create the future path of the output
			$CurrPath = ".\saved_qmgrs\Host_" + $ServerHost + "_-_" + "Qmgr_" + $Qmgr + "\" + $CurrTimeStamp
			
			# If the past does not already exist, then create it
			if ((Test-Path -path $CurrPath) -ne $True){
				New-Item $CurrPath -type directory | out-null
			}

			# Retrive the full path to the current path
			$CurrPath = Resolve-Path $CurrPath

			# Set the full path to the current logfile
			$CurrLog = "$CurrPath" + "\" + $CurrTimeStamp + "_qmgr_" + $Qmgr + "_output.log"
			
			# Log which qmgr and serverhost is currently in use
			$Tmp = " ### ATTEMPTING TO USE SAVEQMGR ON QMGR " + $Qmgr.ToUpper() + " AT SERVERHOST " + $ServerHost.ToUpper() + " ###`r`n"
			log $Tmp
			
			# Log more confirmation regarding which settings are used at the moment
			log "USING THE FOLLOWING SETTINGS:"
			log "QMGR NAME: $Qmgr"
			log "CHANNEL: $Channel"
			log "HOST: $ServerHost"
			log "PORT: $Port"
			log "MCA USER: $Mca"
			log "DESTINATION FOLDER: $CurrPath`r`n"

			# Start defining $SaveQmgrCmds, which will contain all the commands later passed to saveqmgrc
			# Below we set the first commands, which is to set serverhost, port, channel, and qmgr name
			$SaveQmgrCmds = $SaveQmgrcPath + " -a " + $ServerHost + "(" + $Port + ") -x " + $Channel + " -r " + $Qmgr
			
			# See if the optionsfile defined in $OptionsFile is found
			if ((Test-Path -path $OptionsFile) -eq $True){
				# If optionsfile found, then retrive the full path to the file
				$OptionsFile = Resolve-Path $OptionsFile
				
				# Log confirmation that it was found
				log "Able to locate: $OptionsFile"
				log "Since the optionsfile were found, saveqmgrc.exe will now use the option values from there when connecting to qmgr $Qmgr`r`n"
				
				# Get the content of the optionsfile
				[xml] $XmlOptions = Get-Content $OptionsFile

				# Set the Xpath expression for the optionsfile
				$XpathOption = "/Options"
				
				# Iterate the result
				$XmlOptions | select-xml -xpath $XpathOption | foreach {
					# If node outputFileNoHeader contains data then add it to $SaveQmgrCmds
					If ($_.node.outputFileNoHeader){
						$SaveQmgrCmds += " -F """ + $CurrPath + "\" + $CurrTimeStamp + "_qmgr_" + $Qmgr + ".mqsc"""
					}

					# If node noSyncPoint contains data then add it to $SaveQmgrCmds
					If ($_.node.noSyncPoint){
						$SaveQmgrCmds += " -np"
					}

					# If node noSubscriptions contains data then add it to $SaveQmgrCmds
					If ($_.node.noSubscriptions){
						$SaveQmgrCmds += " -ns"
					}

					# If node noSystemObjects contains data then add it to $SaveQmgrCmds
					If ($_.node.noSystemObjects){
						$SaveQmgrCmds += " -s " + $_.node.noSystemObjects
					}
					
					# If node qm contains data then add it to $SaveQmgrCmds
					If ($_.node.qm){
						$SaveQmgrCmds += " -qm " + $_.node.qm
					}
					
					# If node resetChannels contains data then add it to $SaveQmgrCmds
					If ($_.node.resetChannels){
						$SaveQmgrCmds += " -R"
					}

					# If node permDyn contains data then add it to $SaveQmgrCmds
					If ($_.node.permDyn){
						$SaveQmgrCmds += " -p"
					}

					# If node ignoreErrors contains data then add it to $SaveQmgrCmds
					If ($_.node.ignoreErrors){
						$SaveQmgrCmds += " -i"
					}

					# If node quiet contains data then add it to $SaveQmgrCmds
					If ($_.node.quiet){
						$SaveQmgrCmds += " -q"
					}

					# If node oneLine contains data then add it to $SaveQmgrCmds
					If ($_.node.oneLine){
						$SaveQmgrCmds += " -1"
					}

					# If node twoLines contains data then add it to $SaveQmgrCmds
					If ($_.node.twoLines){
						$SaveQmgrCmds += " -2"
					}

					# If node saveSecurity contains data then add it to $SaveQmgrCmds
					If ($_.node.saveSecurity){
						$SaveQmgrCmds += " -z """ + $CurrPath + "\" + $CurrTimeStamp + "_qmgr_" + $Qmgr + ".auth"""
					}

					# If node saveSecurityNoHeader contains data then add it to $SaveQmgrCmds
					If ($_.node.saveSecurityNoHeader){
						$SaveQmgrCmds += " -Z """ + $CurrPath + "\" + $CurrTimeStamp + "_qmgr_" + $Qmgr + ".auth"""
					}

					# If node commandServerQ contains data then add it to $SaveQmgrCmds
					If ($_.node.commandServerQ){
						$SaveQmgrCmds += " -cs " + $_.node.commandServerQ
					}
					
					# If node waitTime contains data then add it to $SaveQmgrCmds
					If ($_.node.waitTime){
						$SaveQmgrCmds += " -w " + $_.node.waitTime
					}
					
					# If node version contains data then add it to $SaveQmgrCmds
					If ($_.node.version){
						$SaveQmgrCmds += " -v " + $_.node.version
					}
				}
			} else {
				# If optionsfile not found, then log confirmation of that
				log "Unable to locate: $OptionsFile"
				log "Since the optionsfile weren't found, saveqmgrc.exe will now use default values when connecting to qmgr $Qmgr`r`n"
			}

			# If outputFileNoHeader was not found in the optionsfile, then add the -f to store the mqsc output
			If (!$SaveQmgrCmds.Contains("-F")){
				# Adding to -f flag to $SaveQmgrCmds
				$SaveQmgrCmds = $SaveQmgrCmds + " -f """ + $CurrPath + "\" + $CurrTimeStamp + "_qmgr_" + $Qmgr + ".mqsc"""
			}

			# Define the name of the batfile which will trigger saveqmgrc
			$BatFileName = $CurrTimeStamp + "_qmgr_" + $Qmgr + "_cmds.bat"
			# Set the path to the batfile which will trigger saveqmgrc
			$BatFileName = "$CurrPath\$BatFileName"
			
			# Export all the commands stored in $SaveQmgrCmds to batfile in $BatFileName
			$SaveQmgrCmds | Out-File $BatFileName -Encoding "ascii"

			# Log confirmation that saveqmgrc is triggered
			log "Executing saveqmgrc script:`r`n$BatFileName`r`n"
			
			# Define process information and store it to $pinfo
			$pinfo = New-Object System.Diagnostics.ProcessStartInfo
			# Define what file to use
			$pinfo.FileName = $BatFileName
			# Define is RedirectStandardError should be saved
			$pinfo.RedirectStandardError = $true
			# Define is RedirectStandardOutput should be saved
			$pinfo.RedirectStandardOutput = $true
			# No idea actually what this one did... :)
			$pinfo.UseShellExecute = $false
			# Define a new process
			$p = New-Object System.Diagnostics.Process
			# Add the information declared from the rows before 
			$p.StartInfo = $pinfo
			# Execute the saveqmgrc script!
			$p.Start() | Out-Null
			# Wait for the process to finish
			$p.WaitForExit()
			# Read in the output given by saveqmgrc
			$output = $p.StandardError.ReadToEnd()

			# If all went well then the output should contain "Writing Queue Manager definition"
			If ($output.Contains("Writing Queue Manager definition")){
				# Log confirmation that all went well
				log "All went well when executing saveqmgrc. This is the info msg returned:`r`n`r`n$output"
			} else {
				# Log confirmation that saveqmgrc returned an error msg
				log "An error occured when executing saveqmgrc. This is the error msg returned:`r`n`r`n$output" "error"
			}
			
			# Log confirmation that the script is done with the current qmgr in use 
			$Tmp = " ### DONE USING SAVEQMGR ON QMGR " + $Qmgr.ToUpper() + " AT SERVERHOST " + $ServerHost.ToUpper() + " ###"
			log $Tmp
			
			# Make $CurrLog null so that no additional information is written to the current outputlog defined in $CurrLog
			$CurrLog = $Null
			
			# Log that fancy divider
			log $Divider
		}
	}
	# If something goes wrong when executing start-saveqmgrc
	catch {
		$ErrorMsg = "Error when executing function start-saveqmgrc`r`n"
		$ErrorMsg += [string]$error[0] + "`r`n`r`n"
		$ErrorMsg += [string]$error[0].InvocationInfo.PositionMessage + "`r`n`r`n"
		log $ErrorMsg "error"
	}
}
# Declare a fancy divider, which will make the output easier to reads
$Divider = "`r`n`r`n---------------------------------------------------------------------------------------`r`n"

# Hardcode path and name of where info are to be logged
$global:MainLog = ".\ps_ms03.log"

# Clear any previous content in $MainLog
"" | Out-File -filepath $MainLog

# Declare $SaveQmgrcPath which will contain the path to saveqmgrc.exe
$SaveQmgrcPath = ".\ms03_win\saveqmgrc.exe"

# Log confirmation that the script is starting
log " ### SAVEQMGR SCRIPT STARTED... ###"

log "Verifying if saveqmgrc.exe has been placed in: $SaveQmgrcPath"
# Start looking for saveqmgrc.exe in $SaveQmgrcPath
if ((Test-Path -path $SaveQmgrcPath) -ne $True){
	# If saveqmgrc.exe is not found
	log "saveqmgrc.exe has not been placed in: $SaveQmgrcPath! Download MS03 and place it in this folder in order to proceed. Script will now exit." "error"
	# Exit the remaning script
	exit
} else {
	# If saveqmgrc.exe is found
	
	# Get the full path to saveqmgrc.exe
	$SaveQmgrcPath = Resolve-Path $SaveQmgrcPath
	# Add quotation marks around the path, incase theres an spaces in the folder names
	$SaveQmgrcPath = """$SaveQmgrcPath"""
	# Log confirmation that saveqmgrc.exe was found
	log "Able to locate saveqmgrc.exe in: $SaveQmgrcPath! Script will now proceed!"
}

# Log a fancy divider
log $Divider

# start-saveqmgrc is the main function of this script. which runs saveqmgrc on defined qmgr's
start-saveqmgrc

# delete-files-and-folders-older-than-X-days will delete files and folder older than x days
# x is defined in that function
delete-files-and-folders-older-than-x-days

# Log a fancy divider
log $Divider

# Log confirmation that the script is finished
log " ### SAVEQMGR SCRIPT FINISHED! ###"